#include <stdio.h>
#include <stdlib.h>
#include <strings.h>

#include "election_def.h"

/****************************************************************/
/*  READ_CMDLINE  :   Read in command line args, fill electionDef structure */
/****************************************************************/
int read_cmdline (int argc, const char * argv[], electionDef *myE) {
    int nprocessed;
    int retval = 0;
    const char *orig = argv[0];
    //    int i;

    if (argc < 4) {
        return -1;
    }
    
    /* ignore argv[0] as it is the name of the executable */
    argv += 1;
    argc -= 1;
    
    /* Now check the arguments.  Increment argv as we go,
        so always look at argv[0] */
    while (argc > 0) {
        
        if (strcmp("-candidates", argv[0]) == 0 && argc > 1)
        {


            myE->candfile = malloc (sizeof (char) * (strlen(argv[1]) + 1));

            strcpy (myE->candfile, argv[1]);
            
            nprocessed = 2;

        }
        else if (strcmp("-parties", argv[0]) == 0 && argc > 1)
        {
            nprocessed = 2;

            myE->partyfile = malloc (sizeof (char) * (strlen(argv[1]) + 1));

            strcpy (myE->partyfile, argv[1]);
            
          
        }
        else if (strcmp("-electorates", argv[0]) == 0 && argc > 1)
        {

            read_electorates(myE, (char *)argv[1]);

            nprocessed = 2;


        }
        else if (strcmp("-ballots", argv[0]) == 0 && argc > 1)
        {
            nprocessed = 2;

            myE->ballotfile = malloc (sizeof (char) * (strlen(argv[1]) + 1));

            strcpy (myE->ballotfile, argv[1]);
            
            
        }
      /*
        else if (strcmp("-vacancies", argv[0]) == 0 && argc > 1)
        {

            myE->vacancyCount = atoi(argv[1]);

            nprocessed = 2;
        }
       */ 
        else {
            printf("Argument not recognised: %s\n", argv[0]);
            nprocessed = 1;
            retval++;
        }
        
        argv += nprocessed;
        argc -= nprocessed;

    }

    argv[0] = orig;
    
    return retval;

}


/***********************************************************/
/*
 *  Input functions - read from text files.
 *
 ***********************************************************/

int read_electorates (electionDef *myE, char *fname) {
    int count = 1;
    int malloced = 10;
    FILE *fin;

    char buff[512];
    int  index = 0;
    char name[255];
    int seats = 0;
    
    fin = fopen(fname, "r");

    if (fin == NULL) {
        error_exit("Can't open file:%s\n",fname);
    }
            
    /* Allocate in advance, will alloc more later if needed */
    myE->electorates = malloc (sizeof(electorateDef) * 10);
        
    while (!feof(fin)){
        get_csv_token(fin,buff);
        if (buff[0]!= (char)NULL){

            index = atoi(buff);
            strncpy(name,get_csv_token(fin,buff), 256);
            seats = atoi(get_csv_token(fin,buff));
            get_csv_token(fin,buff); //colour. not sure what that is for!
            
            if (count >= malloced) {
                myE->electorates = realloc(myE->electorates, sizeof(electorateDef) * (count + 10));
                malloced = count+10;
            }

            strncpy(myE->electorates[count].name, name, 256);
            myE->electorates[count].ecode = index;
            myE->electorates[count].seats = seats;
            
            count++;

        }
    }
        
    myE->totalElectorateCount = count;

    fclose (fin);
    
    return 0;
    
}


int read_ballots (electionDef *myE, char *fname) {
    int count = 0;
    int i;
    FILE *fin;

    char buff[512];
    int batch = 0;
    int pindex = 0;
    int pref = 0;
    int ccode = 0;
    int pcode = 0;
    int rcand = 0;
    
    fin = fopen(fname, "r");

    if (fin == NULL) {
        error_exit("Can't open file:%s\n",fname);
    }

 
    /*fldID,batch,pindex,pref,ccode,pcode,rcand*/

    //init hash table...
    for (i = 0 ; i < TABLE_SIZE ; i++) {
        myE->ballots[i] = NULL;
    }

    
    while (!feof(fin)){
        get_csv_token(fin,buff);
        if (buff[0]!= (char)NULL){

            batch = atoi(get_csv_token(fin,buff));
            pindex = atoi(get_csv_token(fin,buff));
            pref = atoi(get_csv_token(fin,buff));
            ccode = atoi(get_csv_token(fin,buff));
            pcode = atoi(get_csv_token(fin,buff));
            rcand = atoi(get_csv_token(fin,buff));

            // Add stuff here...
            new_entry(myE, batch, pindex, pref, ccode, pcode);

            if (pref == 1)
                count++;

        }
    }

    myE->totalBallotCount = count;

    fclose (fin);
    
    return 0;

}



int read_parties (electionDef *myE, char *fname) {
    int count = 0;
    int malloced = 10;
    FILE *fin;

    char buff[512];
    char name[255], abbrev[256];
    int ecode = 0;
    int pcode = 0;
    int numcand = 0;

    fin = fopen(fname, "r");

    if (fin == NULL) {
        error_exit("Can't open file:%s\n",fname);
    }

    /* Allocate in advance, will alloc more later if needed */
    myE->parties = malloc (sizeof(partyDef) * 10);

    while (!feof(fin)){
        get_csv_token(fin,buff);
        if (buff[0]!= (char)NULL){
            ecode = atoi(get_csv_token(fin,buff));
            pcode = atoi(get_csv_token(fin,buff));
            strncpy(name,get_csv_token(fin,buff), 256);
            strncpy(abbrev,get_csv_token(fin,buff), 256);
            numcand = atoi(get_csv_token(fin,buff));
            

            if (ecode == myE->ecode) {

                if (count >= malloced) {
                    myE->electorates = realloc(myE->electorates, sizeof(electorateDef) * (count + 10));
                    malloced = count+10;
                }

                strncpy(myE->parties[count].name, name, 256);
                strncpy(myE->parties[count].abbrev, abbrev, 256);

                myE->parties[count].pcode = pcode;
                myE->parties[count].numcand = numcand;

                myE->parties[count].candidates = malloc (sizeof(int) * numcand);
                
                count++;
            }
        }
    }

    myE->totalPartyCount = count;

    fclose (fin);
    
    return 0;

}



int read_candidates (electionDef *myE, char *fname) {
    int count = 0;
    int malloced = 10;
    FILE *fin;

    char buff[512];
    int  ecode = 0;
    int  pcode = 0;
    int  ccode = 0;
    char name[255];

    fin = fopen(fname, "r");

    if (fin == NULL) {
        error_exit("Can't open file:%s\n",fname);
    }

    /* Allocate in advance, will alloc more later if needed */
    myE->candidates = malloc (sizeof(candidateDef) * 10);

    while (!feof(fin)){
        get_csv_token(fin,buff);

        if (buff[0]!= (char)NULL){
            ecode = atoi(get_csv_token(fin,buff));
            pcode = atoi(get_csv_token(fin,buff));
            ccode = atoi(get_csv_token(fin,buff));

            strncpy(name,get_csv_token(fin,buff), 255);
            
            if (ecode == myE->ecode) {

                if (count >= malloced) {
                    myE->candidates = realloc(myE->candidates, sizeof(candidateDef) * (count + 10));
                    malloced = count+10;
                }

                strncpy(myE->candidates[count].name, name, 255);
                myE->candidates[count].index = count;
                myE->candidates[count].ccode = ccode;
                myE->candidates[count].pcode = pcode;
                myE->candidates[count].status = limbo;
                myE->candidates[count].voteCount = NULL;
                myE->candidates[count].statusObtainedAtCount = 0;
                
                myE->parties[pcode].candidates[ccode] = count;
                
                count++;

            }
           
            
        }
    }


    
    myE->totalCandidateCount = count;

    myE->orderedCandidates = malloc (sizeof(candidateDef) * count);
    myE->electedCandidates = malloc (sizeof(candidateDef) * count);
    myE->alphaCandidates = malloc (sizeof(candidateDef) * count);

    fclose (fin);
    
    return 0;

}



char* get_csv_token(FILE* csvfile, char* token){
    
    int index = 0;
    int inchar;
    
    //if (feof(csvfile)) return NULL;
    
    //  printf("gct\n");
    
    inchar = fgetc(csvfile);
    // printf("%c:%d\n",(char)inchar,index);
    
    token[index] = (char)inchar;
    while ( (token[index] != ',') && (token[index] != '\n') && (!feof(csvfile)) ){
        //if ( !isspace(token[index]) ){
        index++;
        //}
        inchar = fgetc(csvfile);
        //printf("%c:%d\n",(char)inchar,index);
        token[index] = (char)inchar;
    }
token[index] = (char)NULL;
//printf("gct returns token \"%s\" %d length\n",token,index);
return token;
}


int getIntegerFromUser(int min, int max) {
    int temp = min - 1;
    
    while (temp > max || temp < min) {
        
        fprintf(stderr, "Press a key (%d - %d)\n", min, max);
        
        temp = (int)getchar() - '0';
        
    }
    
    return temp;
    
}

